@@ -108,6 +108,14 @@ config_data_get_notify_with_tray (void)
 	return g_settings_get_boolean (calendar_settings, "notify-with-tray");
 }
 
+gint
+config_data_get_default_snooze_minutes (void)
+{
+	ensure_inited ();
+
+	return g_settings_get_int (calendar_settings, "default-snooze-minutes");
+}
+
 static void
 source_written_cb (GObject *source_object,
                    GAsyncResult *result,
