@@ -249,8 +249,6 @@ e_composer_private_constructed (EMsgComposer *composer)
 	gtk_scrolled_window_set_policy (
 		GTK_SCROLLED_WINDOW (widget),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (
-		GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
 	gtk_widget_set_size_request (widget, -1, GALLERY_INITIAL_HEIGHT);
 	gtk_paned_pack1 (GTK_PANED (container), widget, FALSE, FALSE);
 	priv->gallery_scrolled_window = g_object_ref (widget);
@@ -1111,22 +1109,19 @@ composer_load_signature_cb (EMailSignatureComboBox *combo_box,
 	if (is_html && !html_mode) {
 		WebKitDOMElement *element;
 		gchar *inner_text;
-		gchar *html;
 
 		element = webkit_dom_document_create_element (document, "div", NULL);
 		webkit_dom_html_element_set_inner_html (
 			WEBKIT_DOM_HTML_ELEMENT (element), contents, NULL);
 		inner_text = webkit_dom_html_element_get_inner_text (
 			WEBKIT_DOM_HTML_ELEMENT (element));
-		html = camel_text_to_html (inner_text, 0, 0);
-		if (html) {
-			g_free (contents);
 
-			contents = html;
-			length = strlen (contents);
-		}
-		g_free (inner_text);
-	} else if (!is_html) {
+		g_free (contents);
+		contents = inner_text ? g_strstrip (inner_text) : g_strdup ("");
+		is_html = FALSE;
+	}
+
+	if (!is_html) {
 		gchar *html;
 
 		html = camel_text_to_html (contents, 0, 0);
