@@ -2,18 +2,17 @@
 /*
  * e-table-item.c
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) version 3.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  *
  * Authors:
@@ -49,16 +48,17 @@
 #include "gal-a11y-e-table-item-factory.h"
 #include "gal-a11y-e-table-item.h"
 
-/* workaround for avoiding API breakage */
-#define eti_get_type e_table_item_get_type
-G_DEFINE_TYPE (ETableItem, eti, GNOME_TYPE_CANVAS_ITEM)
+G_DEFINE_TYPE (
+	ETableItem,
+	e_table_item,
+	GNOME_TYPE_CANVAS_ITEM)
 
 #define FOCUSED_BORDER 2
 
 #define d(x)
 
 #if d(!)0
-#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)), g_print ("%s: e_table_item_leave_edit\n", __FUNCTION__))
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)), g_print ("%s: e_table_item_leave_edit\n", G_STRFUNC))
 #else
 #define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)))
 #endif
@@ -78,7 +78,7 @@ enum {
 	CLICK,
 	KEY_PRESS,
 	START_DRAG,
-	STYLE_SET,
+	STYLE_UPDATED,
 	SELECTION_MODEL_REMOVED,
 	SELECTION_MODEL_ADDED,
 	LAST_SIGNAL
@@ -99,6 +99,7 @@ enum {
 	PROP_LENGTH_THRESHOLD,
 	PROP_CURSOR_ROW,
 	PROP_UNIFORM_ROW_HEIGHT,
+	PROP_IS_EDITING,
 
 	PROP_MINIMUM_WIDTH,
 	PROP_WIDTH,
@@ -126,16 +127,16 @@ static void e_table_item_redraw_row (ETableItem *eti, gint row);
  * how much you want their respective channels modified (and in what direction).
  * if it can't do the specified modification, it does it in the oppositon direction */
 static void
-e_hsv_tweak (GdkColor *color,
+e_hsv_tweak (GdkRGBA *rgba,
              gdouble delta_h,
              gdouble delta_s,
              gdouble delta_v)
 {
 	gdouble h, s, v, r, g, b;
 
-	r = color->red   / 65535.0f;
-	g = color->green / 65535.0f;
-	b = color->blue  / 65535.0f;
+	r = rgba->red;
+	g = rgba->green;
+	b = rgba->blue;
 
 	gtk_rgb_to_hsv (r, g, b, &h, &s, &v);
 
@@ -159,30 +160,27 @@ e_hsv_tweak (GdkColor *color,
 
 	gtk_hsv_to_rgb (h, s, v, &r, &g, &b);
 
-	color->red   = r * 65535.0f;
-	color->green = g * 65535.0f;
-	color->blue  = b * 65535.0f;
+	rgba->red = r;
+	rgba->green = g;
+	rgba->blue = b;
 }
 
 inline static gint
 model_to_view_row (ETableItem *eti,
                    gint row)
 {
-	gint i;
 	if (row == -1)
 		return -1;
 	if (eti->uses_source_model) {
-		ETableSubset *etss = E_TABLE_SUBSET (eti->table_model);
-		if (eti->row_guess >= 0 && eti->row_guess < etss->n_map) {
-			if (etss->map_table[eti->row_guess] == row) {
-				return eti->row_guess;
-			}
-		}
-		for (i = 0; i < etss->n_map; i++) {
-			if (etss->map_table[i] == row)
-				return i;
-		}
-		return -1;
+		gint model_row;
+
+		model_row = e_table_subset_view_to_model_row (
+			E_TABLE_SUBSET (eti->table_model), eti->row_guess);
+		if (model_row >= 0 && model_row == row)
+			return eti->row_guess;
+
+		return e_table_subset_model_to_view_row (
+			E_TABLE_SUBSET (eti->table_model), row);
 	} else
 		return row;
 }
@@ -192,26 +190,29 @@ view_to_model_row (ETableItem *eti,
                    gint row)
 {
 	if (eti->uses_source_model) {
-		ETableSubset *etss = E_TABLE_SUBSET (eti->table_model);
-		if (row >= 0 && row < etss->n_map) {
+		gint model_row;
+
+		model_row = e_table_subset_view_to_model_row (
+			E_TABLE_SUBSET (eti->table_model), row);
+
+		if (model_row >= 0)
 			eti->row_guess = row;
-			return etss->map_table[row];
-		} else
-			return -1;
+
+		return model_row;
 	} else
 		return row;
 }
 
 inline static gint
 model_to_view_col (ETableItem *eti,
-                   gint col)
+                   gint model_col)
 {
 	gint i;
-	if (col == -1)
+	if (model_col == -1)
 		return -1;
 	for (i = 0; i < eti->cols; i++) {
 		ETableCol *ecol = e_table_header_get_column (eti->header, i);
-		if (ecol->col_idx == col)
+		if (ecol->spec->model_col == model_col)
 			return i;
 	}
 	return -1;
@@ -219,10 +220,11 @@ model_to_view_col (ETableItem *eti,
 
 inline static gint
 view_to_model_col (ETableItem *eti,
-                   gint col)
+                   gint view_col)
 {
-	ETableCol *ecol = e_table_header_get_column (eti->header, col);
-	return ecol ? ecol->col_idx : -1;
+	ETableCol *ecol = e_table_header_get_column (eti->header, view_col);
+
+	return (ecol != NULL) ? ecol->spec->model_col : -1;
 }
 
 static void
@@ -241,7 +243,7 @@ eti_grab (ETableItem *eti,
           guint32 time)
 {
 	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (eti);
-	d (g_print ("%s: time: %d\n", __FUNCTION__, time));
+	d (g_print ("%s: time: %d\n", G_STRFUNC, time));
 	if (eti->grabbed_count == 0) {
 		GdkGrabStatus grab_status;
 
@@ -263,7 +265,7 @@ eti_grab (ETableItem *eti,
 			eti);
 
 		if (grab_status != GDK_GRAB_SUCCESS) {
-			d (g_print ("%s: gtk_grab_add\n", __FUNCTION__));
+			d (g_print ("%s: gtk_grab_add\n", G_STRFUNC));
 			gtk_grab_add (GTK_WIDGET (item->canvas));
 			eti->gtk_grabbed = TRUE;
 		}
@@ -276,14 +278,14 @@ eti_ungrab (ETableItem *eti,
             guint32 time)
 {
 	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (eti);
-	d (g_print ("%s: time: %d\n", __FUNCTION__, time));
+	d (g_print ("%s: time: %d\n", G_STRFUNC, time));
 	eti->grabbed_count--;
 	if (eti->grabbed_count == 0) {
 		if (eti->grab_cancelled) {
 			eti->grab_cancelled = FALSE;
 		} else {
 			if (eti->gtk_grabbed) {
-				d (g_print ("%s: gtk_grab_remove\n", __FUNCTION__));
+				d (g_print ("%s: gtk_grab_remove\n", G_STRFUNC));
 				gtk_grab_remove (GTK_WIDGET (item->canvas));
 				eti->gtk_grabbed = FALSE;
 			}
@@ -297,7 +299,7 @@ eti_ungrab (ETableItem *eti,
 inline static gboolean
 eti_editing (ETableItem *eti)
 {
-	d (g_print ("%s: %s\n", __FUNCTION__, (eti->editing_col == -1) ? "false":"true"));
+	d (g_print ("%s: %s\n", G_STRFUNC, (eti->editing_col == -1) ? "false":"true"));
 
 	if (eti->editing_col == -1)
 		return FALSE;
@@ -305,85 +307,44 @@ eti_editing (ETableItem *eti)
 		return TRUE;
 }
 
-inline static GdkColor *
+static void
 eti_get_cell_background_color (ETableItem *eti,
                                gint row,
                                gint col,
                                gboolean selected,
-                               gboolean *allocatedp)
+                               GdkRGBA *background)
 {
 	ECellView *ecell_view = eti->cell_views[col];
 	GtkWidget *canvas;
-	GdkColor *background, bg;
-	GtkStyle *style;
 	gchar *color_spec = NULL;
-	gboolean allocated = FALSE;
 
 	canvas = GTK_WIDGET (GNOME_CANVAS_ITEM (eti)->canvas);
-	style = gtk_widget_get_style (canvas);
 
 	if (selected) {
 		if (gtk_widget_has_focus (canvas))
-			background = &style->bg[GTK_STATE_SELECTED];
+			e_utils_get_theme_color (canvas, "theme_selected_bg_color", E_UTILS_DEFAULT_THEME_SELECTED_BG_COLOR, background);
 		else
-			background = &style->bg[GTK_STATE_ACTIVE];
+			e_utils_get_theme_color (canvas, "theme_unfocused_selected_bg_color,theme_selected_bg_color", E_UTILS_DEFAULT_THEME_UNFOCUSED_SELECTED_BG_COLOR, background);
 	} else {
-		background = &style->base[GTK_STATE_NORMAL];
+		e_utils_get_theme_color (canvas, "theme_base_color", E_UTILS_DEFAULT_THEME_BASE_COLOR, background);
 	}
 
 	color_spec = e_cell_get_bg_color (ecell_view, row);
 
 	if (color_spec != NULL) {
-		if (gdk_color_parse (color_spec, &bg)) {
-			background = gdk_color_copy (&bg);
-			allocated = TRUE;
-		}
+		GdkRGBA bg;
+
+		if (gdk_rgba_parse (&bg, color_spec))
+			*background = bg;
 	}
 
 	if (eti->alternating_row_colors) {
 		if (row % 2) {
 
 		} else {
-			if (!allocated) {
-				background = gdk_color_copy (background);
-				allocated = TRUE;
-			}
 			e_hsv_tweak (background, 0.0f, 0.0f, -0.07f);
 		}
 	}
-	if (allocatedp)
-		*allocatedp = allocated;
-
-	return background;
-}
-
-inline static GdkColor *
-eti_get_cell_foreground_color (ETableItem *eti,
-                               gint row,
-                               gint col,
-                               gboolean selected,
-                               gboolean *allocated)
-{
-	GtkWidget *canvas;
-	GdkColor *foreground;
-	GtkStyle *style;
-
-	canvas = GTK_WIDGET (GNOME_CANVAS_ITEM (eti)->canvas);
-	style = gtk_widget_get_style (canvas);
-
-	if (allocated)
-		*allocated = FALSE;
-
-	if (selected) {
-		if (gtk_widget_has_focus (canvas))
-			foreground = &style->fg[GTK_STATE_SELECTED];
-		else
-			foreground = &style->fg[GTK_STATE_ACTIVE];
-	} else {
-		foreground = &style->text[GTK_STATE_NORMAL];
-	}
-
-	return foreground;
 }
 
 static void
@@ -559,8 +520,8 @@ eti_update (GnomeCanvasItem *item,
 	ETableItem *eti = E_TABLE_ITEM (item);
 	gdouble x1, x2, y1, y2;
 
-	if (GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->update)
-		(*GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->update)(item, i2c, flags);
+	if (GNOME_CANVAS_ITEM_CLASS (e_table_item_parent_class)->update)
+		(*GNOME_CANVAS_ITEM_CLASS (e_table_item_parent_class)->update)(item, i2c, flags);
 
 	x1 = item->x1;
 	y1 = item->y1;
@@ -1047,7 +1008,7 @@ eti_show_cursor (ETableItem *eti,
 	cursor_row = e_selection_model_cursor_row (eti->selection);
 #endif
 
-	d (g_print ("%s: cursor row: %d\n", __FUNCTION__, cursor_row));
+	d (g_print ("%s: cursor row: %d\n", G_STRFUNC, cursor_row));
 
 	if (cursor_row != -1) {
 		cursor_row = model_to_view_row (eti, cursor_row);
@@ -1088,11 +1049,11 @@ eti_check_cursor_bounds (ETableItem *eti)
 		return;
 	}
 
-	d (g_print ("%s: model cursor row: %d\n", __FUNCTION__, cursor_row));
+	d (g_print ("%s: model cursor row: %d\n", G_STRFUNC, cursor_row));
 
 	cursor_row = model_to_view_row (eti, cursor_row);
 
-	d (g_print ("%s: cursor row: %d\n", __FUNCTION__, cursor_row));
+	d (g_print ("%s: cursor row: %d\n", G_STRFUNC, cursor_row));
 
 	eti_get_region (
 		eti,
@@ -1104,14 +1065,14 @@ eti_check_cursor_bounds (ETableItem *eti)
 	eti->cursor_y2 = y2;
 	eti->cursor_on_screen = e_canvas_item_area_shown (GNOME_CANVAS_ITEM (eti), x1, y1, x2, y2);
 
-	d (g_print ("%s: cursor on screen: %s\n", __FUNCTION__, eti->cursor_on_screen ? "TRUE" : "FALSE"));
+	d (g_print ("%s: cursor on screen: %s\n", G_STRFUNC, eti->cursor_on_screen ? "TRUE" : "FALSE"));
 }
 
 static void
 eti_maybe_show_cursor (ETableItem *eti,
                        gint delay)
 {
-	d (g_print ("%s: cursor on screen: %s\n", __FUNCTION__, eti->cursor_on_screen ? "TRUE" : "FALSE"));
+	d (g_print ("%s: cursor on screen: %s\n", G_STRFUNC, eti->cursor_on_screen ? "TRUE" : "FALSE"));
 	if (eti->cursor_on_screen)
 		eti_show_cursor (eti, delay);
 	eti_check_cursor_bounds (eti);
@@ -1135,7 +1096,7 @@ eti_idle_show_cursor_cb (gpointer data)
 static void
 eti_idle_maybe_show_cursor (ETableItem *eti)
 {
-	d (g_print ("%s: cursor on screen: %s\n", __FUNCTION__, eti->cursor_on_screen ? "TRUE" : "FALSE"));
+	d (g_print ("%s: cursor on screen: %s\n", G_STRFUNC, eti->cursor_on_screen ? "TRUE" : "FALSE"));
 	if (eti->cursor_on_screen) {
 		g_object_ref (eti);
 		if (!eti->cursor_idle_id)
@@ -1160,7 +1121,7 @@ static void
 eti_freeze (ETableItem *eti)
 {
 	eti->frozen_count++;
-	d (g_print ("%s: %d\n", __FUNCTION__, eti->frozen_count));
+	d (g_print ("%s: %d\n", G_STRFUNC, eti->frozen_count));
 }
 
 static void
@@ -1170,7 +1131,7 @@ eti_unfreeze (ETableItem *eti)
 		return;
 
 	eti->frozen_count--;
-	d (g_print ("%s: %d\n", __FUNCTION__, eti->frozen_count));
+	d (g_print ("%s: %d\n", G_STRFUNC, eti->frozen_count));
 	if (eti->frozen_count == 0 && eti->queue_show_cursor) {
 		eti_show_cursor (eti, 0);
 		eti_check_cursor_bounds (eti);
@@ -1178,6 +1139,24 @@ eti_unfreeze (ETableItem *eti)
 	}
 }
 
+void
+e_table_item_freeze (ETableItem *eti)
+{
+	if (!eti)
+		return;
+
+	eti_freeze (eti);
+}
+
+void
+e_table_item_thaw (ETableItem *eti)
+{
+	if (!eti)
+		return;
+
+	eti_unfreeze (eti);
+}
+
 /*
  * Callback routine: invoked before the ETableModel suffers a change
  */
@@ -1370,9 +1349,9 @@ e_table_item_redraw_range (ETableItem *eti,
 		NULL);
 
 	if ((start_col == cursor_col) ||
-	    (end_col   == cursor_col) ||
+	    (end_col == cursor_col) ||
 	    (view_to_model_row (eti, start_row) == cursor_row) ||
-	    (view_to_model_row (eti, end_row)   == cursor_row))
+	    (view_to_model_row (eti, end_row) == cursor_row))
 		border = 2;
 	else
 		border = 0;
@@ -1432,7 +1411,8 @@ eti_add_table_model (ETableItem *eti,
 
 	if (E_IS_TABLE_SUBSET (table_model)) {
 		eti->uses_source_model = 1;
-		eti->source_model = E_TABLE_SUBSET (table_model)->source;
+		eti->source_model = e_table_subset_get_source_model (
+			E_TABLE_SUBSET (table_model));
 		if (eti->source_model)
 			g_object_ref (eti->source_model);
 	}
@@ -1576,7 +1556,7 @@ eti_dispose (GObject *object)
 	eti->height_cache = NULL;
 
 	/* Chain up to parent's dispose() method. */
-	G_OBJECT_CLASS (eti_parent_class)->dispose (object);
+	G_OBJECT_CLASS (e_table_item_parent_class)->dispose (object);
 }
 
 static void
@@ -1697,6 +1677,9 @@ eti_get_property (GObject *object,
 	case PROP_UNIFORM_ROW_HEIGHT:
 		g_value_set_boolean (value, eti->uniform_row_height);
 		break;
+	case PROP_IS_EDITING:
+		g_value_set_boolean (value, e_table_item_is_editing (eti));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -1704,71 +1687,71 @@ eti_get_property (GObject *object,
 }
 
 static void
-eti_init (ETableItem *eti)
+e_table_item_init (ETableItem *eti)
 {
-	eti->motion_row	       = -1;
-	eti->motion_col	       = -1;
-	eti->editing_col               = -1;
-	eti->editing_row               = -1;
-	eti->height                    = 0;
-	eti->width                     = 0;
-	eti->minimum_width             = 0;
+	eti->motion_row = -1;
+	eti->motion_col = -1;
+	eti->editing_col = -1;
+	eti->editing_row = -1;
+	eti->height = 0;
+	eti->width = 0;
+	eti->minimum_width = 0;
 
-	eti->save_col                  = -1;
-	eti->save_row                  = -1;
-	eti->save_state                = NULL;
+	eti->save_col = -1;
+	eti->save_row = -1;
+	eti->save_state = NULL;
 
-	eti->click_count               = 0;
+	eti->click_count = 0;
 
-	eti->height_cache              = NULL;
-	eti->height_cache_idle_id      = 0;
-	eti->height_cache_idle_count   = 0;
+	eti->height_cache = NULL;
+	eti->height_cache_idle_id = 0;
+	eti->height_cache_idle_count = 0;
 
-	eti->length_threshold          = -1;
-	eti->uniform_row_height        = FALSE;
+	eti->length_threshold = -1;
+	eti->uniform_row_height = FALSE;
 
-	eti->uses_source_model         = 0;
-	eti->source_model              = NULL;
+	eti->uses_source_model = 0;
+	eti->source_model = NULL;
 
-	eti->row_guess                 = -1;
-	eti->cursor_mode               = E_CURSOR_SIMPLE;
+	eti->row_guess = -1;
+	eti->cursor_mode = E_CURSOR_SIMPLE;
 
-	eti->selection_change_id       = 0;
-	eti->selection_row_change_id   = 0;
-	eti->cursor_change_id          = 0;
-	eti->cursor_activated_id       = 0;
-	eti->selection                 = NULL;
+	eti->selection_change_id = 0;
+	eti->selection_row_change_id = 0;
+	eti->cursor_change_id = 0;
+	eti->cursor_activated_id = 0;
+	eti->selection = NULL;
 
-	eti->old_cursor_row            = -1;
+	eti->old_cursor_row = -1;
 
-	eti->needs_redraw              = 0;
-	eti->needs_compute_height      = 0;
+	eti->needs_redraw = 0;
+	eti->needs_compute_height = 0;
 
-	eti->in_key_press              = 0;
+	eti->in_key_press = 0;
 
-	eti->maybe_did_something       = TRUE;
+	eti->maybe_did_something = TRUE;
 
-	eti->grabbed_count             = 0;
-	eti->gtk_grabbed               = 0;
+	eti->grabbed_count = 0;
+	eti->gtk_grabbed = 0;
 
-	eti->in_drag                   = 0;
-	eti->maybe_in_drag             = 0;
-	eti->grabbed                   = 0;
+	eti->in_drag = 0;
+	eti->maybe_in_drag = 0;
+	eti->grabbed = 0;
 
-	eti->grabbed_col               = -1;
-	eti->grabbed_row               = -1;
+	eti->grabbed_col = -1;
+	eti->grabbed_row = -1;
 
-	eti->cursor_on_screen          = FALSE;
-	eti->cursor_x1                 = -1;
-	eti->cursor_y1                 = -1;
-	eti->cursor_x2                 = -1;
-	eti->cursor_y2                 = -1;
+	eti->cursor_on_screen = FALSE;
+	eti->cursor_x1 = -1;
+	eti->cursor_y1 = -1;
+	eti->cursor_x2 = -1;
+	eti->cursor_y2 = -1;
 
-	eti->rows                      = -1;
-	eti->cols                      = -1;
+	eti->rows = -1;
+	eti->cols = -1;
 
-	eti->frozen_count              = 0;
-	eti->queue_show_cursor         = FALSE;
+	eti->frozen_count = 0;
+	eti->queue_show_cursor = FALSE;
 
 	e_canvas_item_set_reflow_callback (GNOME_CANVAS_ITEM (eti), eti_reflow);
 }
@@ -1795,8 +1778,8 @@ eti_realize (GnomeCanvasItem *item)
 {
 	ETableItem *eti = E_TABLE_ITEM (item);
 
-	if (GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->realize)
-		(*GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->realize)(item);
+	if (GNOME_CANVAS_ITEM_CLASS (e_table_item_parent_class)->realize)
+		(*GNOME_CANVAS_ITEM_CLASS (e_table_item_parent_class)->realize)(item);
 
 	eti->rows = e_table_model_row_count (eti->table_model);
 
@@ -1836,7 +1819,7 @@ eti_unrealize (GnomeCanvasItem *item)
 	ETableItem *eti = E_TABLE_ITEM (item);
 
 	if (eti->grabbed_count > 0) {
-		d (g_print ("%s: eti_ungrab\n", __FUNCTION__));
+		d (g_print ("%s: eti_ungrab\n", G_STRFUNC));
 		eti_ungrab (eti, -1);
 	}
 
@@ -1857,14 +1840,14 @@ eti_unrealize (GnomeCanvasItem *item)
 
 	eti->height = 0;
 
-	if (GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->unrealize)
-		(*GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->unrealize)(item);
+	if (GNOME_CANVAS_ITEM_CLASS (e_table_item_parent_class)->unrealize)
+		(*GNOME_CANVAS_ITEM_CLASS (e_table_item_parent_class)->unrealize)(item);
 }
 
 static void
 eti_draw_grid_line (ETableItem *eti,
                     cairo_t *cr,
-                    GtkStyle *style,
+                    const GdkRGBA *rgba,
                     gint x1,
                     gint y1,
                     gint x2,
@@ -1872,11 +1855,11 @@ eti_draw_grid_line (ETableItem *eti,
 {
 	cairo_save (cr);
 
-	cairo_set_line_width (cr, 1.0);
-	gdk_cairo_set_source_color (cr, &style->dark[GTK_STATE_NORMAL]);
+	cairo_set_line_width (cr, 0.5);
+	gdk_cairo_set_source_rgba (cr, rgba);
 
-	cairo_move_to (cr, x1 + 0.5, y1 + 0.5);
-	cairo_line_to (cr, x2 + 0.5, y2 + 0.5);
+	cairo_move_to (cr, x1 - 0.5, y1 + 0.5);
+	cairo_line_to (cr, x2 - 0.5, y2 + 0.5);
 	cairo_stroke (cr);
 
 	cairo_restore (cr);
@@ -1902,9 +1885,12 @@ eti_draw (GnomeCanvasItem *item,
 	cairo_matrix_t i2c;
 	gdouble eti_base_x, eti_base_y, lower_right_y, lower_right_x;
 	GtkWidget *canvas = GTK_WIDGET (item->canvas);
-	GtkStyle *style = gtk_widget_get_style (canvas);
+	GdkRGBA line_color;
 	gint height_extra = eti->horizontal_draw_grid ? 1 : 0;
 
+	e_utils_get_theme_color (canvas, "theme_bg_color", E_UTILS_DEFAULT_THEME_BG_COLOR, &line_color);
+	e_utils_shade_color (&line_color, &line_color, E_UTILS_DARKNESS_MULT);
+
 	/*
 	 * Find out our real position after grouping
 	 */
@@ -1949,8 +1935,8 @@ eti_draw (GnomeCanvasItem *item,
 	 * Compute row span.
 	 */
 	if (eti->uniform_row_height) {
-		first_row = (y          - floor (eti_base_y) - height_extra) / (ETI_ROW_HEIGHT (eti, -1) + height_extra);
-		last_row  = (y + height - floor (eti_base_y)               ) / (ETI_ROW_HEIGHT (eti, -1) + height_extra) + 1;
+		first_row = (y - floor (eti_base_y) - height_extra) / (ETI_ROW_HEIGHT (eti, -1) + height_extra);
+		last_row = (y + height - floor (eti_base_y)               ) / (ETI_ROW_HEIGHT (eti, -1) + height_extra) + 1;
 		if (first_row > last_row)
 			return;
 		y_offset = floor (eti_base_y) - y + height_extra + first_row * (ETI_ROW_HEIGHT (eti, -1) + height_extra);
@@ -1997,7 +1983,7 @@ eti_draw (GnomeCanvasItem *item,
 	f_found = FALSE;
 
 	if (eti->horizontal_draw_grid && first_row == 0)
-		eti_draw_grid_line (eti, cr, style, eti_base_x - x, yd, eti_base_x + eti->width - x, yd);
+		eti_draw_grid_line (eti, cr, &line_color, eti_base_x - x, yd, eti_base_x + eti->width - x, yd);
 
 	yd += height_extra;
 
@@ -2024,15 +2010,14 @@ eti_draw (GnomeCanvasItem *item,
 			gboolean col_selected = selected;
 			gboolean cursor = FALSE;
 			ECellFlags flags;
-			gboolean free_background;
-			GdkColor *background;
+			GdkRGBA background;
 			gint x1, x2, y1, y2;
 			cairo_pattern_t *pat;
 
 			switch (eti->cursor_mode) {
 			case E_CURSOR_SIMPLE:
 			case E_CURSOR_SPREADSHEET:
-				if (cursor_col == ecol->col_idx && cursor_row == view_to_model_row (eti, row)) {
+				if (cursor_col == ecol->spec->model_col && cursor_row == view_to_model_row (eti, row)) {
 					col_selected = !col_selected;
 					cursor = TRUE;
 				}
@@ -2047,24 +2032,24 @@ eti_draw (GnomeCanvasItem *item,
 			x2 = x1 + ecol->width;
 			y2 = yd + height;
 
-			background = eti_get_cell_background_color (eti, row, col, col_selected, &free_background);
+			eti_get_cell_background_color (eti, row, col, col_selected, &background);
 
 			cairo_save (cr);
 			pat = cairo_pattern_create_linear (0, y1, 0, y2);
 			cairo_pattern_add_color_stop_rgba (
-				pat, 0.0, background->red / 65535.0 ,
-				background->green / 65535.0,
-				background->blue / 65535.0, selected ? 0.8: 1.0);
+				pat, 0.0, background.red,
+				background.green,
+				background.blue, selected ? 0.8: 1.0);
 			if (selected)
 				cairo_pattern_add_color_stop_rgba (
-					pat, 0.5, background->red / 65535.0 ,
-					background->green / 65535.0,
-					background->blue / 65535.0, 0.9);
+					pat, 0.5, background.red,
+					background.green,
+					background.blue, 0.9);
 
 			cairo_pattern_add_color_stop_rgba (
-				pat, 1, background->red / 65535.0 ,
-				background->green / 65535.0,
-				background->blue / 65535.0, selected ? 0.8 : 1.0);
+				pat, 1, background.red,
+				background.green,
+				background.blue, selected ? 0.8 : 1.0);
 			cairo_rectangle (cr, x1, y1, ecol->width, height - 1);
 			cairo_set_source (cr, pat);
 			cairo_fill_preserve (cr);
@@ -2074,28 +2059,25 @@ eti_draw (GnomeCanvasItem *item,
 			cairo_restore (cr);
 
 			cairo_save (cr);
-			cairo_set_line_width (cr, 1.0);
+			cairo_set_line_width (cr, 0.5);
 			cairo_set_source_rgba (
-				cr, background->red / 65535.0 ,
-				background->green / 65535.0,
-				background->blue / 65535.0, 1);
-			cairo_move_to (cr, x1, y1);
-			cairo_line_to (cr, x2, y1);
+				cr, background.red,
+				background.green,
+				background.blue, 1);
+			cairo_move_to (cr, x1, y1 + 0.5);
+			cairo_line_to (cr, x2, y1 + 0.5);
 			cairo_stroke (cr);
 
-			cairo_set_line_width (cr, 1.0);
+			cairo_set_line_width (cr, 0.5);
 			cairo_set_source_rgba (
-				cr, background->red / 65535.0 ,
-				background->green / 65535.0,
-				background->blue / 65535.0, 1);
-			cairo_move_to (cr, x1, y2);
-			cairo_line_to (cr, x2, y2);
+				cr, background.red,
+				background.green,
+				background.blue, 1);
+			cairo_move_to (cr, x1, y2 + 0.5);
+			cairo_line_to (cr, x2, y2 + 0.5);
 			cairo_stroke (cr);
 			cairo_restore (cr);
 
-			if (free_background)
-				gdk_color_free (background);
-
 			flags = col_selected ? E_CELL_SELECTED : 0;
 			flags |= gtk_widget_has_focus (canvas) ? E_CELL_FOCUSED : 0;
 			flags |= cursor ? E_CELL_CURSOR : 0;
@@ -2116,8 +2098,11 @@ eti_draw (GnomeCanvasItem *item,
 			}
 
 			e_cell_draw (
-				ecell_view, cr, ecol->col_idx, col, row, flags,
-				xd, yd, xd + ecol->width, yd + height);
+				ecell_view, cr,
+				ecol->spec->model_col,
+				col, row, flags,
+				xd, yd,
+				xd + ecol->width, yd + height);
 
 			if (!f_found && !selected) {
 				switch (eti->cursor_mode) {
@@ -2148,7 +2133,7 @@ eti_draw (GnomeCanvasItem *item,
 		yd += height;
 
 		if (eti->horizontal_draw_grid) {
-			eti_draw_grid_line (eti, cr, style, eti_base_x - x, yd, eti_base_x + eti->width - x, yd);
+			eti_draw_grid_line (eti, cr, &line_color, eti_base_x - x, yd, eti_base_x + eti->width - x, yd);
 			yd++;
 		}
 	}
@@ -2159,7 +2144,7 @@ eti_draw (GnomeCanvasItem *item,
 		for (col = first_col; col <= last_col; col++) {
 			ETableCol *ecol = e_table_header_get_column (eti->header, col);
 
-			eti_draw_grid_line (eti, cr, style, xd, y_offset, xd, yd - 1);
+			eti_draw_grid_line (eti, cr, &line_color, xd, y_offset, xd, yd - 1);
 
 			/*
 			 * This looks wierd, but it is to draw the last line
@@ -2174,17 +2159,22 @@ eti_draw (GnomeCanvasItem *item,
 	 */
 	if (eti->draw_focus && f_found) {
 		static const double dash[] = { 1.0, 1.0 };
+		GdkRGBA bg, fg;
+
+		e_utils_get_theme_color (canvas, "theme_bg_color", E_UTILS_DEFAULT_THEME_BG_COLOR, &bg);
+		e_utils_get_theme_color (canvas, "theme_fg_color", E_UTILS_DEFAULT_THEME_FG_COLOR, &fg);
+
 		cairo_set_line_width (cr, 1.0);
 		cairo_rectangle (
 			cr,
 			f_x1 + 0.5, f_x2 + 0.5,
 			f_x2 - f_x1 - 1, f_y2 - f_y1);
 
-		gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_NORMAL]);
+		gdk_cairo_set_source_rgba (cr, &bg);
 		cairo_stroke_preserve (cr);
 
 		cairo_set_dash (cr, dash, G_N_ELEMENTS (dash), 0.0);
-		gdk_cairo_set_source_color (cr, &style->fg[GTK_STATE_NORMAL]);
+		gdk_cairo_set_source_rgba (cr, &fg);
 		cairo_stroke (cr);
 	}
 }
@@ -2325,7 +2315,7 @@ eti_e_cell_event (ETableItem *item,
 		GdkDevice *event_device;
 		guint32 event_time;
 
-		d (g_print ("%s: eti_grab\n", __FUNCTION__));
+		d (g_print ("%s: eti_grab\n", G_STRFUNC));
 
 		event_device = gdk_event_get_device (event);
 		event_time = gdk_event_get_time (event);
@@ -2338,7 +2328,7 @@ eti_e_cell_event (ETableItem *item,
 	if (actions & E_CELL_UNGRAB) {
 		guint32 event_time;
 
-		d (g_print ("%s: eti_ungrab\n", __FUNCTION__));
+		d (g_print ("%s: eti_ungrab\n", G_STRFUNC));
 
 		event_time = gdk_event_get_time (event);
 		eti_ungrab (item, event_time);
@@ -2391,7 +2381,7 @@ eti_event (GnomeCanvasItem *item,
 		gint new_cursor_row, new_cursor_col;
 		ECellFlags flags = 0;
 
-		d (g_print ("%s: GDK_BUTTON_PRESS received, button %d\n", __FUNCTION__, event_button));
+		d (g_print ("%s: GDK_BUTTON_PRESS received, button %d\n", G_STRFUNC, event_button));
 
 		switch (event_button) {
 		case 1: /* Fall through. */
@@ -2502,13 +2492,13 @@ eti_event (GnomeCanvasItem *item,
 				event_device = gdk_event_get_device (event);
 
 				eti->maybe_in_drag = TRUE;
-				eti->drag_row      = new_cursor_row;
-				eti->drag_col      = new_cursor_col;
-				eti->drag_x        = event_x_item;
-				eti->drag_y        = event_y_item;
-				eti->drag_state    = event_state;
-				eti->grabbed       = TRUE;
-				d (g_print ("%s: eti_grab\n", __FUNCTION__));
+				eti->drag_row = new_cursor_row;
+				eti->drag_col = new_cursor_col;
+				eti->drag_x = event_x_item;
+				eti->drag_y = event_y_item;
+				eti->drag_state = event_state;
+				eti->grabbed = TRUE;
+				d (g_print ("%s: eti_grab\n", G_STRFUNC));
 				eti_grab (eti, event_device, event_time);
 			}
 
@@ -2558,10 +2548,10 @@ eti_event (GnomeCanvasItem *item,
 		gint col, row;
 		gint cursor_row, cursor_col;
 
-		d (g_print ("%s: GDK_BUTTON_RELEASE received, button %d\n", __FUNCTION__, event_button));
+		d (g_print ("%s: GDK_BUTTON_RELEASE received, button %d\n", G_STRFUNC, event_button));
 
 		if (eti->grabbed_count > 0) {
-			d (g_print ("%s: eti_ungrab\n", __FUNCTION__));
+			d (g_print ("%s: eti_ungrab\n", G_STRFUNC));
 			eti_ungrab (eti, event_time);
 		}
 
@@ -2592,7 +2582,7 @@ eti_event (GnomeCanvasItem *item,
 					&col, &row, &x1, &y1);
 				g_print (
 					"%s: find_cell(%f, %f) = %s(%d, %d, %f, %f)\n",
-					__FUNCTION__, event_x_item, event_y_item,
+					G_STRFUNC, event_x_item, event_y_item,
 					cell_found?"true":"false", col, row, x1, y1);
 			}
 #endif
@@ -2608,7 +2598,7 @@ eti_event (GnomeCanvasItem *item,
 
 			if (eti_editing (eti) && cursor_row == view_to_model_row (eti, row) && cursor_col == view_to_model_col (eti, col)) {
 
-				d (g_print ("%s: GDK_BUTTON_RELEASE received, button %d, line: %d\n", __FUNCTION__, event_button, __LINE__))
+				d (g_print ("%s: GDK_BUTTON_RELEASE received, button %d, line: %d\n", G_STRFUNC, event_button, __LINE__))
 ;
 
 				ecell_view = eti->cell_views[col];
@@ -2646,7 +2636,7 @@ eti_event (GnomeCanvasItem *item,
 		gdouble x1, y1;
 #endif
 
-		d (g_print ("%s: GDK_2BUTTON_PRESS received, button %d\n", __FUNCTION__, event_button));
+		d (g_print ("%s: GDK_2BUTTON_PRESS received, button %d\n", G_STRFUNC, event_button));
 
 		/*
 		 * click_count is so that if you click on two
@@ -2693,7 +2683,7 @@ eti_event (GnomeCanvasItem *item,
 			}
 
 			if (eti->grabbed_count > 0) {
-				d (g_print ("%s: eti_ungrab\n", __FUNCTION__));
+				d (g_print ("%s: eti_ungrab\n", G_STRFUNC));
 				eti_ungrab (eti, event_time);
 			}
 
@@ -2718,8 +2708,7 @@ eti_event (GnomeCanvasItem *item,
 		gnome_canvas_item_w2i (item, &event_x_item, &event_y_item);
 
 		if (eti->maybe_in_drag) {
-			if (abs (event_x_item - eti->drag_x) >= 3 ||
-			    abs (event_y_item - eti->drag_y) >= 3) {
+			if (gtk_drag_check_threshold (GTK_WIDGET (item->canvas), eti->drag_x, eti->drag_y, event_x_item, event_y_item)) {
 				gboolean drag_handled;
 
 				eti->maybe_in_drag = 0;
@@ -2792,7 +2781,7 @@ eti_event (GnomeCanvasItem *item,
 		gint cursor_row, cursor_col;
 		gint handled = TRUE;
 
-		d (g_print ("%s: GDK_KEY_PRESS received, keyval: %d\n", __FUNCTION__, (gint) e->key.keyval));
+		d (g_print ("%s: GDK_KEY_PRESS received, keyval: %d\n", G_STRFUNC, (gint) e->key.keyval));
 
 		g_object_get (
 			eti->selection,
@@ -3005,7 +2994,7 @@ eti_event (GnomeCanvasItem *item,
 	case GDK_KEY_RELEASE: {
 		gint cursor_row, cursor_col;
 
-		d (g_print ("%s: GDK_KEY_RELEASE received, keyval: %d\n", __FUNCTION__, (gint) event_keyval));
+		d (g_print ("%s: GDK_KEY_RELEASE received, keyval: %d\n", G_STRFUNC, (gint) event_keyval));
 
 		g_object_get (
 			eti->selection,
@@ -3029,7 +3018,7 @@ eti_event (GnomeCanvasItem *item,
 	case GDK_LEAVE_NOTIFY:
 		d (leave = TRUE);
 	case GDK_ENTER_NOTIFY:
-		d (g_print ("%s: %s received\n", __FUNCTION__, leave ? "GDK_LEAVE_NOTIFY" : "GDK_ENTER_NOTIFY"));
+		d (g_print ("%s: %s received\n", G_STRFUNC, leave ? "GDK_LEAVE_NOTIFY" : "GDK_ENTER_NOTIFY"));
 		if (eti->motion_row != -1 && eti->motion_col != -1)
 			return_val = eti_e_cell_event (
 				eti, eti->cell_views[eti->motion_col],
@@ -3042,7 +3031,7 @@ eti_event (GnomeCanvasItem *item,
 		break;
 
 	case GDK_FOCUS_CHANGE:
-		d (g_print ("%s: GDK_FOCUS_CHANGE received, %s\n", __FUNCTION__, e->focus_change.in ? "in": "out"));
+		d (g_print ("%s: GDK_FOCUS_CHANGE received, %s\n", G_STRFUNC, e->focus_change.in ? "in": "out"));
 		if (event->focus_change.in) {
 			if (eti->save_row != -1 &&
 			    eti->save_col != -1 &&
@@ -3058,26 +3047,28 @@ eti_event (GnomeCanvasItem *item,
 			if (eti_editing (eti)) {
 				eti_free_save_state (eti);
 
-				eti->save_row   = eti->editing_row;
-				eti->save_col   = eti->editing_col;
+				eti->save_row = eti->editing_row;
+				eti->save_col = eti->editing_col;
 				eti->save_state = e_cell_save_state (
 					eti->cell_views[eti->editing_col], view_to_model_col (eti, eti->editing_col),
 					eti->editing_col, eti->editing_row, eti->edit_ctx);
 				e_table_item_leave_edit_(eti);
 			}
 		}
+		return_val = FALSE;
+		break;
 
 	default:
 		return_val = FALSE;
+		break;
 	}
-	/* d(g_print("%s: returning: %s\n", __FUNCTION__, return_val?"true":"false"));*/
+	/* d(g_print("%s: returning: %s\n", G_STRFUNC, return_val?"true":"false"));*/
 
 	return return_val;
 }
 
 static void
-eti_style_set (ETableItem *eti,
-               GtkStyle *previous_style)
+eti_style_updated (ETableItem *eti)
 {
 	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (eti);
 
@@ -3089,7 +3080,7 @@ eti_style_set (ETableItem *eti,
 		gint n_cells = eti->n_cells;
 
 		for (i = 0; i < n_cells; i++) {
-			e_cell_style_set (eti->cell_views[i], previous_style);
+			e_cell_style_updated (eti->cell_views[i]);
 		}
 	}
 
@@ -3104,30 +3095,30 @@ eti_style_set (ETableItem *eti,
 }
 
 static void
-eti_class_init (ETableItemClass *class)
+e_table_item_class_init (ETableItemClass *class)
 {
 	GnomeCanvasItemClass *item_class = GNOME_CANVAS_ITEM_CLASS (class);
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
-	object_class->dispose       = eti_dispose;
-	object_class->set_property  = eti_set_property;
-	object_class->get_property  = eti_get_property;
+	object_class->dispose = eti_dispose;
+	object_class->set_property = eti_set_property;
+	object_class->get_property = eti_get_property;
 
-	item_class->update          = eti_update;
-	item_class->realize         = eti_realize;
-	item_class->unrealize       = eti_unrealize;
-	item_class->draw            = eti_draw;
-	item_class->point           = eti_point;
-	item_class->event           = eti_event;
+	item_class->update = eti_update;
+	item_class->realize = eti_realize;
+	item_class->unrealize = eti_unrealize;
+	item_class->draw = eti_draw;
+	item_class->point = eti_point;
+	item_class->event = eti_event;
 
-	class->cursor_change    = NULL;
+	class->cursor_change = NULL;
 	class->cursor_activated = NULL;
-	class->double_click     = NULL;
-	class->right_click      = NULL;
-	class->click            = NULL;
-	class->key_press        = NULL;
-	class->start_drag       = NULL;
-	class->style_set        = eti_style_set;
+	class->double_click = NULL;
+	class->right_click = NULL;
+	class->click = NULL;
+	class->key_press = NULL;
+	class->start_drag = NULL;
+	class->style_updated = eti_style_updated;
 	class->selection_model_removed = NULL;
 	class->selection_model_added = NULL;
 
@@ -3273,6 +3264,16 @@ eti_class_init (ETableItemClass *class)
 			FALSE,
 			G_PARAM_READWRITE));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_IS_EDITING,
+		g_param_spec_boolean (
+			"is-editing",
+			"Whether is in an editing mode",
+			"Whether is in an editing mode",
+			FALSE,
+			G_PARAM_READABLE));
+
 	eti_signals[CURSOR_CHANGE] = g_signal_new (
 		"cursor_change",
 		G_OBJECT_CLASS_TYPE (object_class),
@@ -3353,15 +3354,14 @@ eti_class_init (ETableItemClass *class)
 		G_TYPE_INT,
 		GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
 
-	eti_signals[STYLE_SET] = g_signal_new (
-		"style_set",
+	eti_signals[STYLE_UPDATED] = g_signal_new (
+		"style_updated",
 		G_OBJECT_CLASS_TYPE (object_class),
 		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETableItemClass, style_set),
+		G_STRUCT_OFFSET (ETableItemClass, style_updated),
 		NULL, NULL,
-		g_cclosure_marshal_VOID__OBJECT,
-		G_TYPE_NONE, 1,
-		GTK_TYPE_STYLE);
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 
 	eti_signals[SELECTION_MODEL_REMOVED] = g_signal_new (
 		"selection_model_removed",
@@ -3573,7 +3573,7 @@ e_table_item_enter_edit (ETableItem *eti,
 	g_return_if_fail (eti != NULL);
 	g_return_if_fail (E_IS_TABLE_ITEM (eti));
 
-	d (g_print ("%s: %d, %d, eti_editing() = %s\n", __FUNCTION__, col, row, eti_editing (eti)?"true":"false"));
+	d (g_print ("%s: %d, %d, eti_editing() = %s\n", G_STRFUNC, col, row, eti_editing (eti)?"true":"false"));
 
 	if (eti_editing (eti))
 		e_table_item_leave_edit_(eti);
@@ -3581,7 +3581,11 @@ e_table_item_enter_edit (ETableItem *eti,
 	eti->editing_col = col;
 	eti->editing_row = row;
 
-	eti->edit_ctx = e_cell_enter_edit (eti->cell_views[col], view_to_model_col (eti, col), col, row);
+	if (col >= 0) {
+		eti->edit_ctx = e_cell_enter_edit (eti->cell_views[col], view_to_model_col (eti, col), col, row);
+
+		g_object_notify (G_OBJECT (eti), "is-editing");
+	}
 }
 
 /**
@@ -3599,7 +3603,7 @@ e_table_item_leave_edit (ETableItem *eti)
 	g_return_if_fail (eti != NULL);
 	g_return_if_fail (E_IS_TABLE_ITEM (eti));
 
-	d (g_print ("%s: eti_editing() = %s\n", __FUNCTION__, eti_editing (eti)?"true":"false"));
+	d (g_print ("%s: eti_editing() = %s\n", G_STRFUNC, eti_editing (eti)?"true":"false"));
 
 	if (!eti_editing (eti))
 		return;
@@ -3616,6 +3620,8 @@ e_table_item_leave_edit (ETableItem *eti)
 		eti->cell_views[col],
 		view_to_model_col (eti, col),
 		col, row, edit_ctx);
+
+	g_object_notify (G_OBJECT (eti), "is-editing");
 }
 
 /**
@@ -3727,13 +3733,13 @@ e_table_item_calculate_print_widths (ETableHeader *eth,
 	expansion = 0;
 	for (i = 0; i < eth->col_count; i++) {
 		extra -= eth->columns[i]->min_width * scale;
-		if (eth->columns[i]->resizable && eth->columns[i]->expansion > 0)
+		if (eth->columns[i]->spec->resizable && eth->columns[i]->expansion > 0)
 			last_resizable = i;
-		expansion += eth->columns[i]->resizable ? eth->columns[i]->expansion : 0;
+		expansion += eth->columns[i]->spec->resizable ? eth->columns[i]->expansion : 0;
 		widths[i] = eth->columns[i]->min_width * scale;
 	}
 	for (i = 0; i <= last_resizable; i++) {
-		widths[i] += extra * (eth->columns[i]->resizable ? eth->columns[i]->expansion : 0) / expansion;
+		widths[i] += extra * (eth->columns[i]->spec->resizable ? eth->columns[i]->expansion : 0) / expansion;
 	}
 
 	return widths;
@@ -4039,3 +4045,32 @@ e_table_item_get_printable (ETableItem *item)
 
 	return printable;
 }
+
+/**
+ * e_table_item_is_editing:
+ * @eti: an %ETableItem 
+ *
+ * Returns: Whether the table item is currently editing cell content.
+ **/
+gboolean
+e_table_item_is_editing (ETableItem *eti)
+{
+	g_return_val_if_fail (E_IS_TABLE_ITEM (eti), FALSE);
+
+	return eti_editing (eti);
+}
+
+/**
+ * e_table_item_cursor_scrolled:
+ * @eti: an %ETableItem 
+ *
+ * Does necessary recalculations after cursor scrolled, like whether
+ * the cursor is on screen or not anymore.
+ **/
+void
+e_table_item_cursor_scrolled (ETableItem *eti)
+{
+	g_return_if_fail (E_IS_TABLE_ITEM (eti));
+
+	eti_check_cursor_bounds (eti);
+}
