@@ -570,6 +570,21 @@ attachment_button_expanded (GObject *object,
 		return;
 	}
 
+	if (WEBKIT_DOM_IS_HTML_ELEMENT (element) && expanded &&
+	    webkit_dom_element_get_child_element_count (element) == 0) {
+		gchar *inner_html_data;
+
+		inner_html_data = webkit_dom_element_get_attribute (element, "inner-html-data");
+		if (inner_html_data && *inner_html_data) {
+			WebKitDOMHTMLElement *html_element;
+
+			html_element = WEBKIT_DOM_HTML_ELEMENT (element);
+			webkit_dom_html_element_set_inner_html (html_element, inner_html_data, NULL);
+		}
+
+		g_free (inner_html_data);
+	}
+
 	/* Show or hide the DIV which contains
 	 * the attachment (iframe, image...). */
 	css = webkit_dom_element_get_style (element);
@@ -814,37 +829,15 @@ mail_display_plugin_widget_requested (WebKitWebView *web_view,
 		/* Bind visibility of DOM element containing related
 		 * attachment with 'expanded' property of this
 		 * attachment button. */
-		WebKitDOMElement *attachment;
-		WebKitDOMDocument *document;
 		EMailPartAttachment *empa = (EMailPartAttachment *) part;
-		gchar *attachment_part_id;
-		gchar *wrapper_element_id;
 
-		if (empa->attachment_view_part_id)
-			attachment_part_id = empa->attachment_view_part_id;
-		else
-			attachment_part_id = part_id;
+		e_attachment_button_set_expandable (E_ATTACHMENT_BUTTON (widget),
+			e_mail_part_attachment_get_expandable (empa));
 
-		/* Find attachment-wrapper div which contains
-		 * the content of the attachment (iframe). */
-		document = webkit_web_view_get_dom_document (
-			WEBKIT_WEB_VIEW (display));
-		wrapper_element_id = g_strconcat (
-			attachment_part_id, ".wrapper", NULL);
-		attachment = find_element_by_id (document, wrapper_element_id);
-		g_free (wrapper_element_id);
-
-		/* None found? Attachment cannot be expanded */
-		if (attachment == NULL) {
-			e_attachment_button_set_expandable (
-				E_ATTACHMENT_BUTTON (widget), FALSE);
-		} else {
+		if (e_mail_part_attachment_get_expandable (empa)) {
 			CamelMimePart *mime_part;
 			const CamelContentDisposition *disposition;
 
-			e_attachment_button_set_expandable (
-				E_ATTACHMENT_BUTTON (widget), TRUE);
-
 			/* Show/hide the attachment when the EAttachmentButton
 			 * is expanded/collapsed or shown/hidden. */
 			g_signal_connect (
