@@ -513,6 +513,10 @@ report_error_to_ui (CamelService *service,
 	g_return_if_fail (CAMEL_IS_SERVICE (service));
 	g_return_if_fail (error != NULL);
 
+	/* Ignore 'offline' errors */
+	if (g_error_matches (error, CAMEL_SERVICE_ERROR, CAMEL_SERVICE_ERROR_UNAVAILABLE))
+		return;
+
 	if (folder_name) {
 		tmp = g_strdup_printf ("%s: %s",
 			camel_service_get_display_name (service),
@@ -1232,6 +1236,7 @@ refresh_folders_exec (struct _refresh_folders_msg *m,
 	gboolean success;
 	gboolean delete_junk = FALSE, expunge = FALSE;
 	GHashTable *known_errors;
+	EMailBackend *mail_backend;
 	GError *local_error = NULL;
 	gulong handler_id = 0;
 
@@ -1240,10 +1245,14 @@ refresh_folders_exec (struct _refresh_folders_msg *m,
 			m->info->cancellable, "cancelled",
 			G_CALLBACK (main_op_cancelled_cb), cancellable);
 
-	success = camel_service_connect_sync (
-		CAMEL_SERVICE (m->store), cancellable, error);
-	if (!success)
+	success = camel_service_connect_sync (CAMEL_SERVICE (m->store), cancellable, &local_error);
+	if (!success) {
+		if (g_error_matches (local_error, CAMEL_SERVICE_ERROR, CAMEL_SERVICE_ERROR_UNAVAILABLE))
+			g_clear_error (&local_error);
+		else
+			g_propagate_error (error, local_error);
 		goto exit;
+	}
 
 	get_folders (m->store, m->folders, m->finfo);
 
@@ -1256,6 +1265,8 @@ refresh_folders_exec (struct _refresh_folders_msg *m,
 		goto exit;
 	}
 
+	mail_backend = E_MAIL_BACKEND (e_shell_get_backend_by_name (e_shell_get_default (), "mail"));
+
 	known_errors = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
 	for (i = 0; i < m->folders->len; i++) {
@@ -1266,6 +1277,10 @@ refresh_folders_exec (struct _refresh_folders_msg *m,
 		if (folder && camel_folder_synchronize_sync (folder, expunge, cancellable, &local_error))
 			camel_folder_refresh_info_sync (folder, cancellable, &local_error);
 
+		if (folder && !local_error && mail_backend) {
+			em_utils_process_autoarchive_sync (mail_backend, folder, m->folders->pdata[i], cancellable, &local_error);
+		}
+
 		if (local_error != NULL) {
 			const gchar *error_message = local_error->message ? local_error->message : _("Unknown error");
 
